@@ -101,6 +101,9 @@ class ContainerLauncher:
         iam_ecs_task_role_arn = None
         iam_ecs_task_execution_role_arn = None
         iam_ec2_role_arn = None
+        logs = None
+        if 'logs' in container_input and 'ecs_task' in container_input['logs']:
+            logs = container_input['logs']['ecs_task']
         if container_input.get('spot') is not None:
             if container_input['spot'].get('max_interruption_frequency') is not None:
                 max_spot_interruption_frequency = container_input['spot']['max_interruption_frequency']
@@ -246,7 +249,7 @@ class ContainerLauncher:
             )
 
             self.__create_ecs_task_definition(ecs_task_definition_data, instance_type, iam_ecs_task_role_arn,
-                                              iam_ecs_task_execution_role_arn)
+                                              iam_ecs_task_execution_role_arn, logs)
 
             self.__dynamodb.update_state(entry_id, self.__resources)
 
@@ -254,13 +257,11 @@ class ContainerLauncher:
                 'commands': commands
             })
 
-            logs = {
-                'log_group': '/aws/batch/job',
-                'log_stream': f"{self.__ECS_TASK_LOG_STREAM_PREFIX}-{self.__options['name_suffix']}"
-            }
-
             self.logger.info(f'ECS task logs: {json.dumps(logs)}')
 
+            if logs:
+                self.__dynamodb.add_ecs_task_logs(entry_id, logs)
+
             self.__dynamodb.update_status(entry_id, RunStatus.IN_PROGRESS.name)
             self.__dynamodb.add_task_arn(entry_id, self.__resources['ecs_cluster_name'], task_arn)
 
@@ -354,6 +355,9 @@ class ContainerLauncher:
             'status': RunStatus.IN_PROGRESS.name
         }
 
+    def get_container_record(self, entry_id):
+        return self.__dynamodb.get_entry(entry_id)
+
     def destroy(self, entry_id):
         self.logger.info('Destroying resources')
         entry = self.__dynamodb.get_entry(entry_id)
@@ -973,7 +977,7 @@ class ContainerLauncher:
         self.logger.debug(f'The cloudwatch alarm {alarm_name} has been created')
 
     def __create_ecs_task_definition(self, task_def_opts, instance_type, task_role_arn=None,
-                                     task_execution_role_arn=None):
+                                     task_execution_role_arn=None, logs=None):
         task_role_name = task_role_arn.split('/')[-1] if task_role_arn is not None else self.__resources[
             'ecs_task_iam_role_name']
         task_execution_role_name = task_execution_role_arn.split('/')[-1] if task_execution_role_arn is not None else \
@@ -990,46 +994,49 @@ class ContainerLauncher:
         tags = list(map(lambda tag: {'key': tag['Key'], 'value': tag['Value']}, self.__tags)) if len(
             self.__tags) > 0 else [{'key': 'package', 'value': 'acl'}]
 
+        container_definition = {
+            'name': 'default',
+            'image': docker_image,
+            'cpu': cpu,
+            'memory': memory,
+            'entryPoint': [
+                '/bin/bash',
+                '-ec'
+            ],
+            'command': commands,
+            'environment': [
+                {
+                    'name': 'string',
+                    'value': 'string'
+                },
+            ],
+            'mountPoints': task_def_opts['containerDefinitions']['mountPoints'] if task_def_opts[
+                'containerDefinitions'].get('mountPoints') else [],
+            'workingDirectory': task_def_opts['containerDefinitions']['workingDirectory'] if task_def_opts[
+                'containerDefinitions'].get('workingDirectory') else '/',
+            'privileged': task_def_opts['containerDefinitions']['privileged'] if task_def_opts[
+                'containerDefinitions'].get('privileged') else False,
+            'linuxParameters': task_def_opts['containerDefinitions']['linuxParameters'] if task_def_opts[
+                'containerDefinitions'].get('linuxParameters') else {}
+        }
+
+        if logs is not None:
+            container_definition['logConfiguration'] = {
+                'logDriver': 'awslogs',
+                'options': {
+                    'awslogs-group': logs['group'],
+                    'awslogs-region': self.__region,
+                    'awslogs-stream-prefix': logs['stream_prefix']
+                }
+            }
+
         response = self.clients['ecs'].register_task_definition(
             family=task_definition_name,
             taskRoleArn=task_role_name,
             executionRoleArn=task_execution_role_name,
             networkMode='host',
             containerDefinitions=[
-                {
-                    # 'user': '0:1001',
-                    'name': 'default',
-                    'image': docker_image,
-                    'cpu': cpu,
-                    'memory': memory,
-                    'entryPoint': [
-                        '/bin/bash',
-                        '-ec'
-                    ],
-                    'command': commands,
-                    'environment': [
-                        {
-                            'name': 'string',
-                            'value': 'string'
-                        },
-                    ],
-                    'mountPoints': task_def_opts['containerDefinitions']['mountPoints'] if task_def_opts[
-                        'containerDefinitions'].get('mountPoints') else [],
-                    'workingDirectory': task_def_opts['containerDefinitions']['workingDirectory'] if task_def_opts[
-                        'containerDefinitions'].get('workingDirectory') else '/',
-                    'privileged': task_def_opts['containerDefinitions']['privileged'] if task_def_opts[
-                        'containerDefinitions'].get('privileged') else False,
-                    'logConfiguration': {
-                        'logDriver': 'awslogs',
-                        'options': {
-                            'awslogs-group': '/aws/batch/job',
-                            'awslogs-region': self.__region,
-                            'awslogs-stream-prefix': f'{self.__ECS_TASK_LOG_STREAM_PREFIX}-{name_suffix}'
-                        },
-                    },
-                    'linuxParameters': task_def_opts['containerDefinitions']['linuxParameters'] if task_def_opts[
-                        'containerDefinitions'].get('linuxParameters') else {}
-                },
+                container_definition
             ],
             volumes=volumes,
             memory=str(memory),
@@ -1084,7 +1091,7 @@ class ContainerLauncher:
 
         timeout_in_minutes = self.__SPOT_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES if is_spot else self.__EC2_INSTANCE_DEPLOYING_TIMEOUT_IN_MINUTES
 
-        self.logger.info(f'Waiting for an EC2 instance deploying ({timeout_in_minutes} minutes)')
+        self.logger.info(f'Waiting for an EC2 instance deploying (timeout: {timeout_in_minutes} minutes)')
 
         while True:
             now_time = datetime.now()
@@ -1160,8 +1167,8 @@ class ContainerLauncher:
         user_data = launch_template_opts.get('user_data', '')
         instance_type = launch_template_opts['instance_type']
         ec2_sg_ids = launch_template_opts['ec2_sg_ids']
-        is_spot = launch_template_opts.get('is_spot', False)
         storages = launch_template_opts.get('storages')
+        block_devices = None
 
         install_ecs_agent_cmd = f'echo ECS_CLUSTER={ecs_cluster_name} >> /etc/ecs/ecs.config;echo ECS_BACKEND_HOST= >> /etc/ecs/ecs.config;'
         if storages is not None:
@@ -1175,11 +1182,12 @@ class ContainerLauncher:
                 if ec2.is_instance_storage_supported_for_instance_type(instance_type):
                     mount_ssd_instance_store_command = '''yum install mdadm -y
 yum install jq -y
-mounts=($(lsblk --fs --json | jq -r '.blockdevices[] | select(.children == null and .fstype == null) | .name'))
-for i in "${mounts[@]}" ; do echo -e "o\\nn\\np\\n1\\n\\n\\nw" | fdisk /dev/$i ; done
+yum install nvme-cli -y
+mounts=($(nvme list -o json  | jq -r '.Devices[] | select(.ModelNumber == "Amazon EC2 NVMe Instance Storage") | .DevicePath'))
+for i in "${mounts[@]}" ; do echo -e "o\\nn\\np\\n1\\n\\n\\nw" | fdisk $i ; done
 mounts_count=${#mounts[@]}
 dev_mounts=""
-for i in "${mounts[@]}" ; do dev_mounts="$dev_mounts /dev/$i" ; done
+for i in "${mounts[@]}" ; do dev_mounts="$dev_mounts $i" ; done
 echo -e "o\\ny" | mdadm --create --verbose --force --auto=yes /dev/md0 --level=0 --raid-devices=$mounts_count $dev_mounts
 mkfs.ext4 /dev/md0
 mkdir %s
@@ -1204,21 +1212,43 @@ mount /dev/md0 %s
                 mount_name = file_system['LustreConfiguration']['MountName']
                 install_fsx_agent_cmd = 'amazon-linux-extras install -y lustre2.10'
                 user_data = f'{user_data}\n{install_fsx_agent_cmd}\nmkdir {fsx_dir}\nmount -t lustre -o noatime,flock {dns_name}@tcp:/{mount_name} {fsx_dir}'
-        # block_devices = None
-        # if storages is not None:
-        #    ebs_storages = [storage for storage in storages if storage['type'] == 'ebs']
-        #    block_devices = []
-        #    for ebs_storage in ebs_storages:
-        #        block_device = {
-        #            'DeviceName': ebs_storage['device_name'],
-        #            'Ebs': {
-        #                'Encrypted': False,
-        #                'DeleteOnTermination': True,
-        #                'VolumeSize': ebs_storage['volume_size'],
-        #                'VolumeType': ebs_storage['volume_type']  # 'standard'|'io1'|'io2'|'gp2'|'sc1'|'st1'|'gp3',
-        #            }
-        #        }
-        #        block_devices.append(block_device)
+            # EBS
+            ebs_storages = [storage for storage in storages if storage['type'] == 'ebs']
+            block_devices = []
+            for ebs_storage in ebs_storages:
+                block_device = {
+                    'DeviceName': ebs_storage['device_name'],
+                    'VirtualName': ebs_storage['device_name'].split('/')[-1],
+                    'Ebs': {
+                        'Encrypted': False,
+                        'DeleteOnTermination': True,
+                        'VolumeSize': ebs_storage['volume_size'],
+                        'VolumeType': ebs_storage['volume_type']  # 'standard'|'io1'|'io2'|'gp2'|'sc1'|'st1'|'gp3',
+                    }
+                }
+                block_devices.append(block_device)
+            if len(ebs_storages) > 0:
+                ebs_storages = ebs_storages[0:1]
+                if len(ebs_storages) == 1:
+                    path = ebs_storages[0].get('dir', '/ebs')
+                    mount_ebs_command = '''yum install mdadm -y
+                    yum install jq -y
+                    yum install nvme-cli -y
+                    mounts=($(nvme list -o json  | jq -r '.Devices[] | select(.ModelNumber == "Amazon Elastic Block Store" and .DevicePath != "/dev/nvme0n1") | .DevicePath'))
+                    echo -e "o\\nn\\np\\n1\\n\\n\\nw" | fdisk ${mounts[0]}
+                    mkfs.ext4 ${mounts[0]}
+                    mkdir %s && mount ${mounts[0]} %s
+                    ''' % (path, path)
+                else:
+                    mount_ebs_command = '''yum install mdadm -y
+                    yum install jq -y
+                    yum install nvme-cli -y
+                    mounts=($(nvme list -o json  | jq -r '.Devices[] | select(.ModelNumber == "Amazon Elastic Block Store") | .DevicePath'))
+                    for i in "${mounts[@]}" ; do echo -e "o\\nn\\np\\n1\\n\\n\\nw" | fdisk /dev/$i ; done
+                    for i in "${mounts[@]}" ; do mkfs.ext4 $i ; done
+                    for i in "${mounts[@]}" ; do device_path=$i && device_path_parts=(${device_path//// }) && mkdir /${device_path_parts[1]} && mount $i /${device_path_parts[1]} ; done
+                                    '''
+                user_data = f'{user_data}\n{mount_ebs_command}'
 
         output_logs_command = 'exec > >(tee /var/log/user-data.log|logger -t user-data -s 2>/dev/console) 2>&1'
         user_data = f'#!/bin/bash\n{install_ecs_agent_cmd}\n{output_logs_command}\n{user_data}'
@@ -1243,17 +1273,12 @@ mount /dev/md0 %s
             ]
         }
 
-        # if block_devices is not None and len(block_devices) > 0:
-        #    launch_template_data['BlockDeviceMappings'] = block_devices
+        if block_devices is not None and len(block_devices) > 0:
+            launch_template_data['BlockDeviceMappings'] = block_devices
 
         if ec2_sg_ids is not None and len(ec2_sg_ids) > 0:
             launch_template_data['SecurityGroupIds'] = ec2_sg_ids
 
-        # if is_spot:
-        #    launch_template_data['InstanceMarketOptions'] = {
-        #        'MarketType': 'spot'
-        #    }
-
         self.clients['ec2'].create_launch_template(
             LaunchTemplateName=launch_template_name,
             LaunchTemplateData=launch_template_data
